#include <vector>
#include <iostream>
#include <string>
#include <CGAL/Mesh_triangulation_3.h>
#include <CGAL/Mesh_complex_3_in_triangulation_3.h>
#include <CGAL/Mesh_criteria_3.h>
#include <CGAL/make_mesh_3.h>
#include <CGAL/Image_3.h>
#include <CGAL/Exact_predicates_inexact_constructions_kernel.h>
#include <CGAL/Mesh_domain_with_polyline_features_3.h>
#include <CGAL/Polyhedron_3.h>
#include <CGAL/Labeled_mesh_domain_3.h>
#include "initialize_triangulation_from_labeled_image.h"  // copied over
#include <CGAL/ImageIO.h>
#include <CGAL/Random.h>


typedef CGAL::Exact_predicates_inexact_constructions_kernel K;
typedef CGAL::Labeled_mesh_domain_3<K> Image_domain;
typedef CGAL::Mesh_domain_with_polyline_features_3<Image_domain> Mesh_domain;
#ifdef CGAL_CONCURRENT_MESH_3
typedef CGAL::Parallel_tag Concurrency_tag;
#else
typedef CGAL::Sequential_tag Concurrency_tag;
#endif
// Triangulation
typedef CGAL::Mesh_triangulation_3<Mesh_domain,CGAL::Default,Concurrency_tag>::type Tr;
typedef CGAL::Mesh_complex_3_in_triangulation_3<Tr> C3t3;
// Criteria
typedef CGAL::Mesh_criteria_3<Tr> Mesh_criteria;

typedef K::Point_3 Point_3;
typedef unsigned char Word_type;

using namespace CGAL::parameters;
#include <CGAL/Mesh_3/polylines_to_protect.h> // undocumented header

int main(int argc, char* argv[])
{
  CGAL::Random random(12345); // Set random seed
  const std::string fname = "cut_section.inr";
  const double tol = 1e-7;
  // Loads image
  CGAL::Image_3 image;
  if(!image.read(fname)){
    std::cerr << "Error: Cannot read file " <<  fname << std::endl;
    return EXIT_FAILURE;
  }
  // Domain
  std::cout << "Setting tol to " << tol << std::endl;
  Mesh_domain domain = Mesh_domain::create_labeled_image_mesh_domain(image, relative_error_bound=tol);
  std::vector<std::vector<Point_3> > polylines_on_bbox;
  CGAL::polylines_to_protect<Point_3, Word_type>(image, polylines_on_bbox);
  domain.add_features(polylines_on_bbox.begin(), polylines_on_bbox.end());


  Mesh_criteria criteria(edge_size=0.35,
                         facet_angle=30,
                         facet_size=0.35,
                         facet_distance=0.025,
                         cell_radius_edge_ratio=3,
                         cell_size=0.35
                         );
  std::cout << "initialize_triangulation_from_labeled_image" << std::endl;
  C3t3 c3t3;
  initialize_triangulation_from_labeled_image(c3t3,
          domain,
          image,
          criteria,
          static_cast<unsigned char>(0),
          true);
  CGAL::refine_mesh_3(c3t3, domain, criteria);

  // Extract surface
  CGAL::Polyhedron_3<K> poly;
  CGAL::facets_in_complex_3_to_triangle_mesh(c3t3, poly);

  std::ofstream mout("surface.off");
  mout << poly;
  mout.close();

  return 0;
}
